//
// YuckFu! (c) Dominic Szablewski aka. cha0s
// www.chaosquake.de/yuckfu
// webmaster@chaosquake.de

#pragma comment(lib,"wininet.lib") 
#include "tinyptc.h"
#include <windows.h> // timer and input
#include <wininet.h> // internet open

#define WIDTH		320	// screenwidth
#define HEIGHT		240	// screenheight
#define SIZE		WIDTH*HEIGHT	// size of pixel[]
#define TITLE		"YuckFu!" // window title

#define TILESIZE	16	// background tile size
#define MAPWIDTH	WIDTH/TILESIZE	// bg map width
#define MAPHEIGHT	HEIGHT/TILESIZE	// bg map height
#define FONTSIZE	8	// font tile size
#define FONTTILES	42	// number of font tiles
#define MAXTILES	20	// max map tiles
#define MAXSPRITES	5	// max sprites
#define MAXPICS		9	// max sprite pics
#define BGLAYERS	1	// max bglayers

#define PLAYERSPEED 40	// player speed
#define PLIMIT		128	// player speed limit
#define GRAVITY		25	// gravity
#define MAXMULTI	10	// max score multiplier
#define TIMESCALE	2.5	// overall game speed

#define RGB24(r,g,b) ((r<<16)+(g<<8)+b)

typedef unsigned char  u8;
typedef unsigned short u16;
typedef unsigned long  u32;



//function prototypes
//game
void init(void);
void spawnfuckit(void);
void putfuckit(void);
void checkfuckit(void);
void colide(void);
void getinput(void);
void updplayerpos(void);

void titlescreen(void);
void pause(void);
void writedebug(void);

//engine
void starttimer(void);
void utdiff(void);

void loadbgmap(u8 *name, u8 layer);
void loadbgtiles(u8 *name, u16 y_tiles);
void loadfont(u8 *name);
void loadpic(u8 id, u8 *name, u8 x_size, u8 y_size);

void putbg(u8 layer);
void putfont(void);
void putsprites(void);
void cls(u32 color);
void cleartext(void);
void redraw_all(void);

//misc
u8 menu(u8 textpos_x, u8 textpos_y, u8 numitems);
void write(u8 textpos_x, u8 textpos_y, u8* text);
void writeint(u8 textpos_x, u8 textpos_y, u32 number, u8 digitcount);
void type(u8 textpos_x, u8 textpos_y, u8* dest); //max 12 chars
//void uploadscore(u32 score, u8* name); //taken out for security



struct grafix {
	u32 px[65535];	//256*256px (u16)
	u16 xsize;
	u16 ysize;
};

struct sprites {
	double xpos;
	double ypos;
	u8 picid;
	u8 inuse;
};


//grafik/map arrays
struct sprites s[MAXSPRITES];
struct grafix pic[MAXPICS];

u32 bg_tiles[MAXTILES][TILESIZE*TILESIZE];
u8 bg_map[BGLAYERS][MAPWIDTH][MAPHEIGHT];

u32 font_tiles[FONTTILES][FONTSIZE*FONTSIZE];
u8 font_map[(WIDTH/FONTSIZE)*(HEIGHT/FONTSIZE)];

static u32 pixel[WIDTH*HEIGHT];


//timer specific variables
__int64 t_freq, t_last;
double t_diff;

//player/gameplay specific
double player_xvec;
double player_yvec;
u32 score;
u8 multiplier;
double level;		//(start)time a fuckit stays on screen
double levelcount;
double fuckitcount;

u8 debug=0;		//show debug info
u8 keylock[255];		//key lock
extern int fullscreen;


int main(void)
{
	u32 i;
	
	// it's a bit naive check the command line like this, but it's simple ..
	if(strstr(GetCommandLine(), " -w"))
		fullscreen=0;
		

	//open window
	if (!ptc_open(TITLE,WIDTH,HEIGHT)) return 1;

	// clear sprites
	for(i=0; i<MAXSPRITES; i++)
		s[i].inuse=0;

	// load sprites/pics/maps
	// sprites and pics are separeted - different sprites can share one pic
	loadpic(0, "BOT", 16, 16);
	loadpic(1, "THRUSTL", 8, 8);
	loadpic(2, "THRUSTR", 8, 8);
	loadpic(3, "THRUSTD", 24, 10);

	loadpic(4, "TILE1", 16, 16);
	loadpic(5, "TILE4", 16, 16);
	loadpic(6, "TILE3", 16, 16);
	loadpic(7, "TILE2", 16, 16);
	loadpic(8, "TILE5", 16, 16);

	loadbgtiles("SOLID", 19);
	loadbgmap("BG1",0);
	loadfont("font_green");

	//give sprite atrb
	//bot
	s[0].inuse=1;
	s[0].picid=0;

	//thrust left
	s[1].picid=1;

	//thrust right
	s[2].picid=2;

	//thrust down
	s[3].inuse=1;
	s[3].picid=3;

	//fuckit
	s[4].inuse=1;

	starttimer();
	utdiff();
	titlescreen();
	utdiff();
	srand((int)(t_diff*100));
	init();

	//main game loop
	while (!GetAsyncKeyState(VK_ESCAPE))
    	{
		utdiff();

		getinput();
		updplayerpos();
		checkfuckit();
		colide();

		//begin output
		if(debug) writedebug();
		writeint(32,0,(int) score,6);
		writeint(36,1,(int) multiplier,2);

		redraw_all();
    	}
	return 0;
}


// function: initialize
// purpose: (re)set game variables
//==========================================================================
void init(void)
{
	s[0].xpos=152;
	s[0].ypos=112;
	player_xvec=0;
	player_yvec=-25;
	score=0;
	multiplier=1;

	level=4;
	levelcount=0;
	fuckitcount=0;
	keylock[VK_RETURN]=1;
	loadbgmap("BG1",0);

	cleartext();
	write(26, 0, "score:");
	write(26, 1, "multi:");
	spawnfuckit();
	utdiff();	
}


// function: spawn fuckit
// purpose: randomly chooses a position to spawn next fuckit
//==========================================================================
void spawnfuckit(void)
{
	u16 xpos,ypos;
	do 	{
		xpos=rand()%18+1;
		ypos=rand()%13+1;
	} while(bg_map[0][xpos][ypos]!=0); // position allready occupied ?

	s[4].picid=8;
	s[4].xpos=xpos*TILESIZE;
	s[4].ypos=ypos*TILESIZE;
}


// function: put fuckit
// purpose: puts a fuckit to bgmap - makes it solid
//==========================================================================
void putfuckit(void)
{
	bg_map[0][(int)s[4].xpos/16][(int)s[4].ypos/16]=5;
	multiplier=1; // reset score multiplier
	spawnfuckit();
}

// function: check fuckit
// purpose: controls fuckit spawntime and checks if it's taken
//==========================================================================
void checkfuckit(void)
{
	//fuckit stat
	levelcount+=t_diff;
	if(levelcount > level)
	{
		levelcount=0;
		if(s[4].picid <=4)
			putfuckit();
		else
			s[4].picid--;
	}

	//"colide" with player
	if((s[0].xpos > s[4].xpos-16)&&(s[0].xpos < s[4].xpos+16)&&
		(s[0].ypos > s[4].ypos-16)&&(s[0].ypos < s[4].ypos+16))
	{
		fuckitcount++;
		levelcount=0;
		score+=(s[4].picid-3)*(s[4].picid-3)*multiplier; //status * multi
		level=100/(fuckitcount+25);  //decrease level
		spawnfuckit();

		if(multiplier < MAXMULTI)
			multiplier++;
	}
}


// function: colide
// purpose: checks player colision with bgmap
//==========================================================================
void colide(void)
{
	u8 name[12];
		
	if((bg_map[0][(int)(s[0].xpos+2)/16][(int)(s[0].ypos+2)/16]>4) ||
	 (bg_map[0][(int)(s[0].xpos+14)/16][(int)(s[0].ypos+14)/16]>4) ||
	 (bg_map[0][(int)(s[0].xpos+2)/16][(int)(s[0].ypos+14)/16]>4) ||
	 (bg_map[0][(int)(s[0].xpos+14)/16][(int)(s[0].ypos+2)/16]>4))
	{
		// show end screen
		write(16,15, "fuckit !");
		write(16,18, "again ?");
		write(16,19, "upload score");
		redraw_all();

		if (menu(15,18, 2)) 
		{
			write(12,15, "enter your name:");
			write(15,18, "         ");
			write(15,19, "             ");
			type(12,16, name);
			write(12,15, "uploading. pleas wait ...");

			redraw_all();

			//uploadscore(score, name);
		}

		//reset the game
		init();
	}	
}


// function: get input
// purpose: handles user input
//==========================================================================
void getinput(void)
{
	if(GetAsyncKeyState(VK_LMENU) && GetAsyncKeyState(VK_RETURN))
		ptc_switch_fs(TITLE, WIDTH, HEIGHT, pixel);

	if(GetAsyncKeyState(VK_LEFT))
	{
		player_xvec-=t_diff*PLAYERSPEED;
		s[2].xpos=s[0].xpos+15 + player_xvec*t_diff;
		s[2].ypos=s[0].ypos+5 + player_yvec*t_diff;

		s[2].inuse ^=1;
	}
	else
		s[2].inuse=0;

	if(GetAsyncKeyState(VK_RIGHT))
	{
		player_xvec+=t_diff*PLAYERSPEED;
		s[1].xpos=s[0].xpos-7 + player_xvec*t_diff;
		s[1].ypos=s[0].ypos+5 + player_yvec*t_diff;
		s[1].inuse ^=1;		
	}
	else
		s[1].inuse=0;


	if(GetAsyncKeyState(VK_UP))
	{
		player_yvec-=t_diff*PLAYERSPEED*1.5;
		s[3].xpos=s[0].xpos-4 + player_xvec*t_diff;
		s[3].ypos=s[0].ypos+12 + player_yvec*t_diff;

		s[3].inuse ^=1;
	}
	else
		s[3].inuse=0;

	if(GetAsyncKeyState(VK_RETURN) && !keylock[VK_RETURN] && !GetAsyncKeyState(VK_LMENU))
	{
		pause();
		keylock[VK_RETURN]=1;
	}
	else if(!GetAsyncKeyState(VK_RETURN))
		keylock[VK_RETURN]=0;

	// toggle debug
	if(GetAsyncKeyState(VK_F2)&&(!keylock[VK_F2]))
	{
		if(debug)
		{
			debug=0;
			write(31, 27, "         ");
			write(29, 28, "           ");
			write(29, 29, "           ");
		}
		else
		{
			debug=1;
			write(31, 27, "fpx:");
			write(29, 28, "x pos:");
			write(29, 29, "y pos:");
		}
		keylock[VK_F2]=1;
	}
	else if(!GetAsyncKeyState(VK_F2))
		keylock[VK_F2]=0;
}


// function: update player position
// purpose: checks speed of player and sets it to new x,y
//==========================================================================
void updplayerpos(void)
{
	player_yvec+=GRAVITY*t_diff;

	//player speed limit
	if(player_xvec>PLIMIT)
		player_xvec=PLIMIT;
	if(player_yvec>PLIMIT)
		player_yvec=PLIMIT;
	if(player_xvec<-PLIMIT)
		player_xvec=-PLIMIT;
	if(player_yvec<-PLIMIT)
		player_yvec=-PLIMIT;

	s[0].ypos+=player_yvec*t_diff;
	s[0].xpos+=player_xvec*t_diff;
}


// function: titlescreen
// purpose: shows titlescreen
//==========================================================================
void titlescreen(void)
{
	write(16,4, "YuckFu!");
	write(4,7, "programmed by cha0s");
	write(5,8, "www.chaosquake.de");
	write(4,10, "fullscreen switch by l0ki");
	write(5,11, "www.dfleischhacker.de");
	write(4,13, "thanks to:");
	write(5,14, "F3:raeuber");
	write(5,15, "F3:neo");
	write(5,16, "F3:ribbler");
	write(10,20, "press enter to play");

    	while (!GetAsyncKeyState(VK_RETURN) || GetAsyncKeyState(VK_LMENU))
	{
		if(GetAsyncKeyState(VK_LMENU) && GetAsyncKeyState(VK_RETURN))
			ptc_switch_fs(TITLE, WIDTH, HEIGHT, pixel);
		
		cls(0x1F2329);
		putbg(0);
		putfont();
		ptc_update(pixel);

		keylock[VK_RETURN]=1;
	}
}


// function: pause (on enter)
// purpose: pause
//==========================================================================
void pause(void)
{
	write(16,15, "paused !");
	keylock[VK_RETURN]=1;
    	while (!(GetAsyncKeyState(VK_RETURN)  && !keylock[VK_RETURN] && !GetAsyncKeyState(VK_LMENU)))
	{
		utdiff();
		putfont();
		ptc_update(pixel);
		if(GetAsyncKeyState(VK_LMENU) && GetAsyncKeyState(VK_RETURN))
			ptc_switch_fs(TITLE, WIDTH, HEIGHT, pixel);
		if(!GetAsyncKeyState(VK_RETURN))
			keylock[VK_RETURN]=0;
	}
	write(16,15, "        ");
}


// function: write debug
// purpose: shows debug information in game
//==========================================================================
void writedebug()
{
	u16 framerate;
	framerate=(int) (1/(t_diff/TIMESCALE));

	writeint(35,27, framerate,4);
	writeint(35,28,(int) s[0].xpos,4);
	writeint(35,29,(int) s[0].ypos,4);
}









// =========================================================================
// main engine functions ===================================================
// =========================================================================


// function: starttimer
// purpose: gets timer frequency
//==========================================================================
void starttimer(void)
{
	LARGE_INTEGER perfreq;
	QueryPerformanceFrequency(&perfreq);
	t_freq= perfreq.QuadPart;
}


// function: update timer difference
// purpose: updates t_diff for time-based movement
//==========================================================================
void utdiff(void)
{
	__int64 t_cur;
	LARGE_INTEGER pertime;

	QueryPerformanceCounter(&pertime);
	t_cur= pertime.QuadPart;
	t_diff=((((double)t_cur-(double)t_last)*TIMESCALE)/(double)t_freq);
	t_last= pertime.QuadPart;
}


// function: load background map
// purpose: loads bg map to array
//==========================================================================
void loadbgmap(u8 *name, u8 layer)
{
	HRSRC hResInfo;
	HGLOBAL hResource;
	u8 *point=NULL;
	u16 bg_x,bg_y;

	hResInfo = FindResource(NULL, name, "MAPS");
	hResource =LoadResource(NULL, hResInfo);
	point = ((u8*)LockResource(hResource));

	for (bg_x=0; bg_x < MAPWIDTH; bg_x++)
		for(bg_y=0; bg_y < MAPHEIGHT; bg_y++)
			bg_map[layer][bg_x][bg_y]=point[bg_y*20+bg_x];
}


// function: load background tiles
// purpose: slices and saves an raw-image into bg_tiles[]
//==========================================================================
void loadbgtiles(u8 *name, u16 y_tiles)
{
	HRSRC hResInfo;
	HGLOBAL hResource;
	u8 *point=NULL;
	u16 tile, px;
	u8 red,green,blue;

	hResInfo = FindResource(NULL, name, "TILES");
	hResource =LoadResource(NULL, hResInfo);
	point = ((u8*)LockResource(hResource));

	for(tile=0; tile < y_tiles; tile++)
	{
		for(px=0; px < TILESIZE*TILESIZE; px++)
		{
			red=*point;
			point++;
			green=*point;
			point++;
			blue=*point;
			point++;
			bg_tiles[tile][px]=RGB24(red,green,blue);
		}
	}
}


// function: load font
// purpose: slices and saves an raw-image into font_tiles[]
//==========================================================================
void loadfont(u8 *name)
{
	HRSRC hResInfo;
	HGLOBAL hResource;
	u8 *point=NULL;
	u16 tile, px;
	u8 red,green,blue;

	hResInfo = FindResource(NULL, name, "TILES");
	hResource =LoadResource(NULL, hResInfo);
	point = ((u8*)LockResource(hResource));

	for(tile=0; tile < FONTTILES; tile++)
	{
		for(px=0; px < FONTSIZE*FONTSIZE; px++)
		{
			red=*point;
			point++;
			green=*point;
			point++;
			blue=*point;
			point++;
			font_tiles[tile][px]=RGB24(red,green,blue);
		}
	}
}


// function: load pic
// purpose: loads a sprite pic from raw-image
//==========================================================================
void loadpic(u8 id, u8 *name, u8 x_size, u8 y_size)
{
	HRSRC hResInfo;
	HGLOBAL hResource;
	u8 *point=NULL;
	u16 i;
	u8 red,green,blue;

	hResInfo = FindResource(NULL, name, "SPRITES");
	hResource =LoadResource(NULL, hResInfo);
	point = ((u8*)LockResource(hResource));

	for(i=0; i< x_size*y_size; i++)
	{
		red=*point;
		point++;
		green=*point;
		point++;
		blue=*point;
		point++;
		pic[id].px[i]=RGB24(red,green,blue); //(red*256*256)+(green*256)+blue;
	}

	pic[id].xsize=x_size;
	pic[id].ysize=y_size;
}


// function: put background
// purpose: prints background to pixel[]
//==========================================================================
void putbg(u8 layer)
{
	u16 bg_x, bg_y, px_x, px_y;

	for(bg_x=0; bg_x < MAPWIDTH; bg_x++) //tile y
		for(bg_y=0; bg_y < MAPHEIGHT; bg_y++) //tile x
			if(bg_map[layer][bg_x][bg_y] !=0) //is there something to show ?
				for(px_y=0; px_y <TILESIZE; px_y++) //pixel y
					for(px_x=0; px_x <TILESIZE; px_x++) //pixel x
						if(!(bg_tiles[ bg_map[layer][bg_x][bg_y] -1 ][px_y*TILESIZE+px_x]==0xFF00FF)) //transparency
							pixel[bg_y*TILESIZE*WIDTH+bg_x*TILESIZE + px_y*WIDTH+px_x] = bg_tiles[ bg_map[layer][bg_x][bg_y]-1][px_y*TILESIZE+px_x];


}


// function: put font
// purpose: prints font (top layer) to pixel[]
//==========================================================================
void putfont(void)
{
	u16 bg_x, bg_y, px_x, px_y;

	for(bg_y=0; bg_y < HEIGHT/FONTSIZE; bg_y++) //tile y
		for(bg_x=0; bg_x < WIDTH/FONTSIZE; bg_x++) //tile x
			if(font_map[bg_y*(WIDTH/FONTSIZE)+bg_x]!=0)  //is there something to show ?
				for(px_y=0; px_y <FONTSIZE; px_y++) //pixel y
					for(px_x=0; px_x <FONTSIZE; px_x++) //pixel x
						if(!(font_tiles[ font_map[bg_y*(WIDTH/FONTSIZE)+bg_x] -1 ][px_y*FONTSIZE+px_x]==0xFF00FF))  // transparency
							pixel[bg_y*FONTSIZE*WIDTH+bg_x*FONTSIZE + px_y*WIDTH+px_x] = font_tiles[ font_map[bg_y*(WIDTH/FONTSIZE)+bg_x] -1 ][px_y*FONTSIZE+px_x];

}


// function: put sprites
// purpose: prints all sprites to pixel[]
//==========================================================================
void putsprites(void)
{
	u16 i,j;
	u8 id;
	for(id=0; id <MAXSPRITES; id++)
		if(s[id].inuse)
			for(i=0; i < pic[s[id].picid].ysize; i++) //y-loop
				for(j=0;j<pic[s[id].picid].xsize; j++) //x-loop
					if(!(pic[s[id].picid].px[i* pic[ s[id].picid ].xsize+j]==0xff00ff)) //transparency
						if (((s[id].ypos+i)*WIDTH+s[id].xpos+j < WIDTH*HEIGHT) && ((s[id].ypos+i)*WIDTH+s[id].xpos+j > 0)) //clipping
							pixel[(i+(int) s[id].ypos )*WIDTH+j+(int) s[id].xpos ] = pic[s[id].picid].px[i* pic[ s[id].picid ].xsize+j];

}


// function: clear screen
// purpose: sets bg color to u32 color
//==========================================================================
void cls(u32 color)
{
	u32 i;
	for(i=0;i<HEIGHT*WIDTH; i++)
		pixel[i]=color;
}


// function: clear text
// purpose: clears text layer
//==========================================================================
void cleartext (void)
{
	u16 i;
	//clear fontmap and sprites
	for(i=0; i<(WIDTH/FONTSIZE)*(HEIGHT/FONTSIZE); i++)
		font_map[i]=0;
}


// function: redraw all
// purpose: puts all sprites and layers and updates screen
//==========================================================================
void redraw_all(void)
{
	cls(0x1F2329);
	putbg(0);
	putsprites();
	putfont();
	ptc_update(pixel);
}








// =========================================================================
// misc functions ==========================================================
// =========================================================================


// function: menu
// purpose: provides cursormovement in a menu, returns menu item
//==========================================================================
u8 menu(u8 textpos_x, u8 textpos_y, u8 numitems)
{
	u8 item=0;
	while (!(GetAsyncKeyState(VK_RETURN) && !GetAsyncKeyState(VK_LMENU)))
	{
		if(GetAsyncKeyState(VK_DOWN) && (item < (numitems-1)) && !keylock[VK_DOWN]) 
		{
			write(textpos_x,textpos_y, " ");
			textpos_y++;
			item++,
			keylock[VK_DOWN]=1;
		}
		else if(GetAsyncKeyState(VK_UP) && (item > 0) && !keylock[VK_UP]) 
		{
			write(textpos_x,textpos_y, " ");
			textpos_y--;
			item--,
			keylock[VK_UP]=1;
		}
		else
		{
			keylock[VK_UP]=0;
			keylock[VK_DOWN]=0;
		}
		write(textpos_x,textpos_y, ">");
		redraw_all();
	}

	keylock[VK_RETURN]=1;
	return item;
}


// function: 	write
// purpose :	writes text to font_map
// ===================================================================================
void write(u8 textpos_x, u8 textpos_y, u8* text)
{
	/*======'ASCII -> BMP Table' =============
	char	bmp-pos		ascii-pos   ->	charsub
	A-Z 	15-40		65-90 		49
	a-z 	15-40		97-112 		81
	0-9	    0-9			48-58		48
	?		10			63			53
	!		11			33			22
	.		12			46			34
	,		13			44			31
	:		14			58			44
	>		42			62			20
	=========================================*/

	u8 charsub, i;

	for(i=0; text[i] != '\0'; i++)
	{
		if (text[i] == 32) 					// space
			charsub= 32;
		else if (text[i] == 33)				// !
			charsub= 21;
		else if (text[i] == 58)				// :
			charsub= 43;
		else if (text[i] == 44)				// ,
			charsub= 30;
		else if (text[i] == 46)				// .
			charsub= 33;
		else if ((text[i] >= 65) & (text[i] <= 90))	// A-Z
			charsub= 49;
		else if ((text[i] >= 97) & (text[i] <= 122))	// a-z
			charsub= 81;
		else if (text[i] == 63)				// ?
			charsub= 52;
		else if ((text[i] >= 48) & (text[i] <= 58))	// 0-9
			charsub= 47;
		else if (text[i] == 62)				// >
			charsub=20;

		font_map[textpos_y *(WIDTH/FONTSIZE) +textpos_x +i]=text[i] -charsub;
	}
}


// function: write integer
// purpose: extracts digits of an int and writes it to font_map
//==========================================================================
void writeint(u8 textpos_x, u8 textpos_y, u32 number, u8 digitcount)
{
	u8 i;
	u8 digits[10]={0,0,0,0,0,0,0,0,0,0};
	for(i=digitcount; i > 0; i--)
	{
		digits[i]=(u8)(number %10);
		number /=10;
		font_map[textpos_y *(WIDTH/FONTSIZE) +textpos_x +i]=digits[i]+1;
	}
}


// function: type
// purpose: get char string from keyboard
//==========================================================================
void type(u8 textpos_x, u8 textpos_y, u8 *dest)
{
	u8 pos=0;
	u8 text[]="             ";
	u8 i;
	while (!(GetAsyncKeyState(VK_RETURN)  && !keylock[VK_RETURN] && !GetAsyncKeyState(VK_LMENU) && (pos>0)))
	{
		//0-9, A-Z
		for(i='0'; i <= 'Z'; i++)
		{
			if(i==':')
				i='A';
			if(GetAsyncKeyState(i) && !keylock[i] && (pos<13))
			{
				keylock[i]=1;
				text[pos]=i;
				pos++;
			}
			else if(!GetAsyncKeyState((u8) i))
				keylock[i]=0;
		}

		//space
		if(GetAsyncKeyState(VK_SPACE) && !keylock[VK_SPACE] && (pos<13))
		{
			keylock[VK_SPACE]=1;
			text[pos]=' ';
			pos++;			
		}
		else if(!GetAsyncKeyState(VK_SPACE))
			keylock[VK_SPACE]=0;

		//backspace
		if(GetAsyncKeyState(VK_BACK) && !keylock[VK_BACK])
		{
			keylock[VK_BACK]=1;
			if(pos>0)
			{
				text[pos]=' ';
				pos--;
				text[pos]=' ';
			}
		}
		else if(!GetAsyncKeyState(VK_BACK))
			keylock[VK_BACK]=0;

		if(!GetAsyncKeyState(VK_RETURN))
			keylock[VK_RETURN]=0;


		write(textpos_x, textpos_y, text);
		redraw_all();
	}

	//clear input echo
	write(textpos_x, textpos_y, "             ");
	strcpy(dest,text);
}










